'use strict';

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var _require = require('bin-protocol'),
    createProtocol = _require.createProtocol;

var _require2 = require('./commands'),
    commands = _require2.commands;

var _ = require('lodash');

var NextionProtocol = createProtocol(function () {
  var reset = this.reader.reset;
  this.reader.reset = function (buf) {
    if (buf.slice(buf.length - 3).equals(Buffer.from([0xff, 0xff, 0xff]))) {
      return reset.call(this, buf.slice(0, -3));
    }
    return reset.call(this, buf);
  };
});

var definitions = {
  byte: {
    read(name) {
      this.UInt8(name);
    }
  },
  response: {
    read() {
      this.commandName();
      var command = this.context.command;

      if (!_.isFunction(this[command])) {
        throw new ReferenceError(`Unknown command "${command}"`);
      }
      this[command]('data');
      return {
        command,
        data: this.context.data
      };
    }
  },
  commandName: {
    read() {
      this.byte('command_value');
      if (!commands[this.context.command_value]) {
        throw new Error('Unknown command');
      }
      this.context.command = commands[this.context.command_value];
    }
  },
  touchEvent: {
    read() {
      this.byte('page_id').byte('button_id').byte('release_event');
      this.context.release_event = Boolean(this.context.release_event);
    }
  },
  pageId: {
    read() {
      this.byte('page_id');
    }
  },
  touchCoordinate: {
    read() {
      this.byte('x_high').byte('x_low').byte('y_high').byte('y_low').touchEvent();
    }
  },
  wake: {
    read(name) {
      this.touchCoordinate(name);
    }
  },
  stringData: {
    read() {
      this.context.value = this.buffer.toString();
    }
  },
  numericData: {
    read() {
      this.Int16LE('value');
    }
  },
  autoSleep: {
    read() {}
  },
  autoWake: {
    read() {}
  },
  cardUpgrade: {
    read() {}
  },
  transmitFinished: {
    read() {}
  },
  transmitReady: {
    read() {}
  },
  startup: {
    read() {}
  }
};

(0, _keys2.default)(definitions).forEach(function (name) {
  NextionProtocol.define(name, definitions[name]);
});

exports.NextionProtocol = NextionProtocol;